@@ -1,3 +1,5 @@
+#![allow(clippy::return_self_not_must_use)]
+
 use super::base::{decode_function_data, AbiError};
 use ethers_core::{
     abi::{AbiDecode, AbiEncode, Detokenize, Function, InvalidOutputType, Tokenizable},
@@ -61,7 +63,7 @@ pub enum ContractError<M: Middleware> {
     ContractNotDeployed,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 #[must_use = "contract calls do nothing unless you `send` or `call` them"]
 /// Helper for managing a transaction before submitting it to a node
 pub struct ContractCall<M, D> {
@@ -75,6 +77,18 @@ pub struct ContractCall<M, D> {
     pub(crate) datatype: PhantomData<D>,
 }
 
+impl<M, D> Clone for ContractCall<M, D> {
+    fn clone(&self) -> Self {
+        ContractCall {
+            tx: self.tx.clone(),
+            function: self.function.clone(),
+            block: self.block,
+            client: self.client.clone(),
+            datatype: self.datatype,
+        }
+    }
+}
+
 impl<M, D: Detokenize> ContractCall<M, D> {
     /// Sets the `from` field in the transaction to the provided value
     pub fn from<T: Into<Address>>(mut self, from: T) -> Self {
