@@ -150,13 +150,23 @@ use std::{fmt::Debug, marker::PhantomData, sync::Arc};
 /// [`Abigen` builder]: crate::Abigen
 /// [`event`]: method@crate::Contract::event
 /// [`method`]: method@crate::Contract::method
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct Contract<M> {
     base_contract: BaseContract,
     client: Arc<M>,
     address: Address,
 }
 
+impl<M> Clone for Contract<M> {
+    fn clone(&self) -> Self {
+        Contract {
+            base_contract: self.base_contract.clone(),
+            client: self.client.clone(),
+            address: self.address,
+        }
+    }
+}
+
 impl<M: Middleware> Contract<M> {
     /// Creates a new contract from the provided client, abi and address
     pub fn new(address: Address, abi: impl Into<BaseContract>, client: impl Into<Arc<M>>) -> Self {
@@ -247,6 +257,7 @@ impl<M: Middleware> Contract<M> {
     /// Returns a new contract instance at `address`.
     ///
     /// Clones `self` internally
+    #[must_use]
     pub fn at<T: Into<Address>>(&self, address: T) -> Self
     where
         M: Clone,
@@ -259,13 +270,12 @@ impl<M: Middleware> Contract<M> {
     /// Returns a new contract instance using the provided client
     ///
     /// Clones `self` internally
-    pub fn connect(&self, client: Arc<M>) -> Self
+    #[must_use]
+    pub fn connect<N>(&self, client: Arc<N>) -> Contract<N>
     where
-        M: Clone,
+        N: Clone,
     {
-        let mut this = self.clone();
-        this.client = client;
-        this
+        Contract { base_contract: self.base_contract.clone(), client, address: self.address }
     }
 
     /// Returns the contract's address
