@@ -1,3 +1,5 @@
+#![allow(clippy::return_self_not_must_use)]
+
 use crate::{log::LogMeta, stream::EventStream, ContractError, EthLogDecode};
 
 use ethers_core::{
@@ -184,7 +186,7 @@ where
         Ok(events)
     }
 
-    fn parse_log(&self, log: Log) -> Result<D, ContractError<M>> {
+    pub fn parse_log(&self, log: Log) -> Result<D, ContractError<M>> {
         D::decode_log(&RawLog { topics: log.topics, data: log.data.to_vec() }).map_err(From::from)
     }
 }
