@@ -1,14 +1,20 @@
 #![cfg(feature = "abigen")]
 //! Test cases to validate the `abigen!` macro
-use ethers_contract::{abigen, EthEvent};
+use ethers_contract::{abigen, EthCall, EthEvent};
 use ethers_core::{
     abi::{AbiDecode, AbiEncode, Address, Tokenizable},
     types::{transaction::eip2718::TypedTransaction, Eip1559TransactionRequest, U256},
+    utils::Ganache,
 };
-use ethers_providers::Provider;
+use ethers_middleware::SignerMiddleware;
+use ethers_providers::{MockProvider, Provider};
+use ethers_signers::{LocalWallet, Signer};
 use ethers_solc::Solc;
 use std::{convert::TryFrom, sync::Arc};
 
+fn assert_codec<T: AbiDecode + AbiEncode>() {}
+fn assert_tokenizeable<T: Tokenizable>() {}
+
 #[test]
 fn can_gen_human_readable() {
     abigen!(
@@ -22,6 +28,11 @@ fn can_gen_human_readable() {
     assert_eq!("ValueChanged(address,string,string)", ValueChangedFilter::abi_signature());
 }
 
+#[test]
+fn can_gen_not_human_readable() {
+    abigen!(VerifierAbiHardhatContract, "./tests/solidity-contracts/verifier_abi_hardhat.json");
+}
+
 #[test]
 fn can_gen_human_readable_multiple() {
     abigen!(
@@ -54,18 +65,24 @@ fn can_gen_structs_readable() {
     ]"#,
         event_derives(serde::Deserialize, serde::Serialize)
     );
-    let value = Addresses {
+    let addr = Addresses {
         addr: vec!["eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee".parse().unwrap()],
         s: "hello".to_string(),
     };
-    let token = value.clone().into_token();
-    assert_eq!(value, Addresses::from_token(token).unwrap());
+    let token = addr.clone().into_token();
+    assert_eq!(addr, Addresses::from_token(token).unwrap());
 
     assert_eq!("ValueChanged", ValueChangedFilter::name());
     assert_eq!(
         "ValueChanged((address,string),(address,string),(address[],string))",
         ValueChangedFilter::abi_signature()
     );
+
+    assert_codec::<Value>();
+    assert_codec::<Addresses>();
+    let encoded = addr.clone().encode();
+    let other = Addresses::decode(&encoded).unwrap();
+    assert_eq!(addr, other);
 }
 
 #[test]
@@ -83,9 +100,10 @@ fn can_gen_structs_with_arrays_readable() {
         "ValueChanged((address,string),(address,string),(address[],string)[])",
         ValueChangedFilter::abi_signature()
     );
-}
 
-fn assert_tokenizeable<T: Tokenizable>() {}
+    assert_codec::<Value>();
+    assert_codec::<Addresses>();
+}
 
 #[test]
 fn can_generate_internal_structs() {
@@ -97,6 +115,10 @@ fn can_generate_internal_structs() {
     assert_tokenizeable::<VerifyingKey>();
     assert_tokenizeable::<G1Point>();
     assert_tokenizeable::<G2Point>();
+
+    assert_codec::<VerifyingKey>();
+    assert_codec::<G1Point>();
+    assert_codec::<G2Point>();
 }
 
 #[test]
@@ -119,6 +141,10 @@ fn can_generate_internal_structs_multiple() {
     assert_tokenizeable::<G1Point>();
     assert_tokenizeable::<G2Point>();
 
+    assert_codec::<VerifyingKey>();
+    assert_codec::<G1Point>();
+    assert_codec::<G2Point>();
+
     let (provider, _) = Provider::mocked();
     let client = Arc::new(provider);
 
@@ -153,6 +179,7 @@ fn can_gen_human_readable_with_structs() {
         event_derives(serde::Deserialize, serde::Serialize)
     );
     assert_tokenizeable::<Foo>();
+    assert_codec::<Foo>();
 
     let (client, _mock) = Provider::mocked();
     let contract = SimpleContract::new(Address::default(), Arc::new(client));
@@ -293,7 +320,7 @@ async fn can_handle_underscore_functions() {
     let compiled = compiled.get(path, contract).unwrap();
     let factory = ethers_contract::ContractFactory::new(
         compiled.abi.unwrap().clone(),
-        compiled.bin.unwrap().clone(),
+        compiled.bytecode().unwrap().clone(),
         client.clone(),
     );
     let addr = factory.deploy("hi".to_string()).unwrap().legacy().send().await.unwrap().address();
@@ -350,6 +377,23 @@ fn can_handle_unique_underscore_functions() {
     let _contract_call = ConsoleLogCalls::Log2(call);
 }
 
+#[test]
+fn can_handle_underscore_numeric() {
+    abigen!(
+        Test,
+        r#"[
+            _100pct(string)
+        ]"#
+    );
+    let _call = _100PctCall("message".to_string());
+
+    let provider = Arc::new(Provider::new(MockProvider::new()));
+    let contract = Test::new(Address::default(), Arc::clone(&provider));
+    // NOTE: this seems to be weird behaviour of `Inflector::to_snake_case` which turns "100pct" ->
+    // "10_0pct"
+    let _call = contract._10_0pct("hello".to_string());
+}
+
 #[test]
 fn can_handle_duplicates_with_same_name() {
     abigen!(
@@ -375,3 +419,174 @@ fn can_handle_duplicates_with_same_name() {
 fn can_abigen_console_sol() {
     abigen!(Console, "ethers-contract/tests/solidity-contracts/console.json",);
 }
+
+#[test]
+fn can_generate_nested_types() {
+    abigen!(
+        Test,
+        r#"[
+        struct Outer {Inner inner; uint256[] arr;}
+        struct Inner {uint256 inner;}
+        function myfun(Outer calldata a)
+    ]"#,
+    );
+
+    assert_eq!(MyfunCall::abi_signature(), "myfun(((uint256),uint256[]))");
+
+    let (client, _mock) = Provider::mocked();
+    let contract = Test::new(Address::default(), Arc::new(client));
+
+    let inner = Inner { inner: 100u64.into() };
+    let a = Outer { inner, arr: vec![101u64.into()] };
+    let _ = contract.myfun(a.clone());
+
+    let call = MyfunCall { a: a.clone() };
+    let encoded_call = contract.encode("myfun", (a,)).unwrap();
+    assert_eq!(encoded_call, call.clone().encode().into());
+    let decoded_call = MyfunCall::decode(encoded_call.as_ref()).unwrap();
+    assert_eq!(call, decoded_call);
+}
+
+#[test]
+fn can_handle_different_calls() {
+    abigen!(
+        Test,
+        r#"[
+        function fooBar()
+        function FOO_BAR()
+    ]"#,
+    );
+
+    let (client, _mock) = Provider::mocked();
+    let contract = Test::new(Address::default(), Arc::new(client));
+
+    let _ = contract.fooBar();
+    let _ = contract.FOO_BAR();
+}
+
+#[test]
+fn can_handle_case_sensitive_calls() {
+    abigen!(
+        StakedOHM,
+        r#"[
+        index()
+        INDEX()
+    ]"#,
+    );
+
+    let (client, _mock) = Provider::mocked();
+    let contract = StakedOHM::new(Address::default(), Arc::new(client));
+
+    let _ = contract.index();
+    let _ = contract.INDEX();
+}
+
+#[tokio::test]
+async fn can_deploy_greeter() {
+    abigen!(Greeter, "ethers-contract/tests/solidity-contracts/greeter.json",);
+    let ganache = ethers_core::utils::Ganache::new().spawn();
+    let from = ganache.addresses()[0];
+    let provider = Provider::try_from(ganache.endpoint())
+        .unwrap()
+        .with_sender(from)
+        .interval(std::time::Duration::from_millis(10));
+    let client = Arc::new(provider);
+
+    let greeter_contract =
+        Greeter::deploy(client, "Hello World!".to_string()).unwrap().legacy().send().await.unwrap();
+
+    let greeting = greeter_contract.greet().call().await.unwrap();
+    assert_eq!("Hello World!", greeting);
+}
+
+#[tokio::test]
+async fn can_abiencoderv2_output() {
+    abigen!(AbiEncoderv2Test, "ethers-contract/tests/solidity-contracts/abiencoderv2test_abi.json",);
+    let ganache = ethers_core::utils::Ganache::new().spawn();
+    let from = ganache.addresses()[0];
+    let provider = Provider::try_from(ganache.endpoint())
+        .unwrap()
+        .with_sender(from)
+        .interval(std::time::Duration::from_millis(10));
+    let client = Arc::new(provider);
+
+    let contract = "AbiencoderV2Test";
+    let path = "./tests/solidity-contracts/Abiencoderv2Test.sol";
+    let compiled = Solc::default().compile_source(path).unwrap();
+    let compiled = compiled.get(path, contract).unwrap();
+    let factory = ethers_contract::ContractFactory::new(
+        compiled.abi.unwrap().clone(),
+        compiled.bytecode().unwrap().clone(),
+        client.clone(),
+    );
+    let addr = factory.deploy(()).unwrap().legacy().send().await.unwrap().address();
+
+    let contract = AbiEncoderv2Test::new(addr, client.clone());
+    let person = Person { name: "Alice".to_string(), age: 20u64.into() };
+
+    let res = contract.default_person().call().await.unwrap();
+    assert_eq!(res, person);
+}
+
+#[test]
+fn can_gen_multi_etherscan() {
+    abigen!(
+        MyContract, "etherscan:0xdAC17F958D2ee523a2206206994597C13D831ec7";
+        MyContract2, "etherscan:0x8418bb725b3ac45ec8fff3791dd8b4e0480cc2a2";
+    );
+
+    let provider = Arc::new(Provider::new(MockProvider::new()));
+    let _contract = MyContract::new(Address::default(), Arc::clone(&provider));
+    let _contract = MyContract2::new(Address::default(), provider);
+}
+
+#[test]
+fn can_gen_reserved_word_field_names() {
+    abigen!(
+        Test,
+        r#"[
+        struct Foo { uint256 ref; }
+    ]"#,
+    );
+
+    let _foo = Foo { ref_: U256::default() };
+}
+
+#[test]
+fn can_handle_overloaded_events() {
+    abigen!(
+        SimpleContract,
+        r#"[
+            event ActionPaused(string cToken, string action, bool pauseState)
+            event ActionPaused(string action, bool pauseState)
+    ]"#
+    );
+
+    let _ev1 = ActionPaused1Filter {
+        c_token: "ctoken".to_string(),
+        action: "action".to_string(),
+        pause_state: false,
+    };
+    let _ev2 = ActionPaused2Filter { action: "action".to_string(), pause_state: false };
+}
+
+#[tokio::test]
+#[cfg(not(feature = "celo"))]
+async fn can_send_struct_param() {
+    abigen!(StructContract, "./tests/solidity-contracts/StructContract.json");
+
+    let server = Ganache::default().spawn();
+    let wallet: LocalWallet = server.keys()[0].clone().into();
+    let provider = Provider::try_from(server.endpoint()).unwrap();
+    let client = Arc::new(SignerMiddleware::new(provider, wallet.with_chain_id(1337u64)));
+
+    let contract = StructContract::deploy(client, ()).unwrap().legacy().send().await.unwrap();
+
+    let point = Point { x: 1337u64.into(), y: 0u64.into() };
+    let tx = contract.submit_point(point).legacy();
+    let tx = tx.send().await.unwrap().await.unwrap().unwrap();
+    assert_eq!(tx.logs.len(), 1);
+
+    let logs: Vec<NewPointFilter> = contract.event().from_block(0u64).query().await.unwrap();
+    assert_eq!(logs.len(), 1);
+}
