@@ -33,7 +33,8 @@ pub fn compile_contract(name: &str, filename: &str) -> (Abi, Bytes) {
     let path = format!("./tests/solidity-contracts/{}", filename);
     let compiled = Solc::default().compile_source(&path).unwrap();
     let contract = compiled.get(&path, name).expect("could not find contract");
-    (contract.abi.unwrap().clone(), contract.bin.unwrap().clone())
+    let (abi, bin, _) = contract.into_parts_or_default();
+    (abi, bin)
 }
 
 /// connects the private key to http://localhost:8545
